package alc.appnaranja.modelo;


import java.util.ArrayList;

import com.google.android.gms.maps.model.LatLng;

import android.graphics.Bitmap;


public interface IModelo {
	/**
	 * Obtiene la lista de peluqueras
	 */
	public void obtenerListaPeluquerias();
	
	/**
	 * Obtiene la descripcin de la peluquera
	 * @param id_peluqueria Identificador de la peluquera
	 * @return Descripcin de la peluquera
	 */
	public String obtenerDescripcionPeluqueria(String id_peluqueria);
	
	/**
	 * Obtiene la imagen de la peluquera
	 * @param id_peluqueria Identificador de la peluquera
	 * @return Imagen de la peluquera.
	 */
	public Bitmap obtenerImagenPeluqueria(String id_peluqueria);
	
	/**
	 * Obtiene la ubicacin geogrfica de la peluquera.
	 * @param id_peluqueria Identificador de la peluquera.
	 * @return Posicin geogrfica de la peluquera.
	 */
	public LatLng posicionPeluqueria(String id_peluqueria);
	
	/**
	 * Obtiene la ubicacin geogrfica de todas las peluqueras.
	 * @return Todas las posiciones geogrficas de las peluqueras.
	 */
	public ArrayList<LatLng> posicionTodasPeluquerias();
	
	/**
	 * Guarda en el modelo los datos introducidos por el usuario.
	 * @param nombre Nombre del usuario.
	 * @param telefono Telfono del usuario, utilizado como identificador del mismo.
	 * @param sexo Sexo del usuario.
	 * @param peluqueria Peluquera elegida por el usuario.
	 */
	public void guardarDatos(String nombre, String telefono, String sexo, String peluqueria);
	
	/**
	 * Obtiene los servicios
	 */
	public void obtenerServicios();
	
	/**
	 * Calcula el precio total segun los servicios.
	 * @param servicios Listado de servicios.
	 * @return Precio de los servicios seleccionados.
	 */
	public String calculaPrecioTotal(ArrayList<String> servicios);
	
	/**
	 * Guarda los datos introducidos por el usuario.
	 * @param servicios Listado de servicios introducidos por el usuario.
	 */
	public void guardarDatos(ArrayList<String> servicios);
	
	
	/**
	 * Obtiene las fechas disponibles para pedir cita.
	 * @return Fechas disponibles para pedir cita.
	 */
	public ArrayList<String> obtenerFechas(String id_peluqueria);
	

	/**
	 * Obtiene las horas disponibles para una fecha determinada.
	 * @param fecha Fecha para buscar las horas.
	 * @return Listado con todas las horas disponibles.
	 */
	public ArrayList<String> obtenerHoras(String fecha);
	
	/**
	 * Guarda los datos introducidos por el usuario.
	 * @param fecha Fecha seleccionada por el usuario
	 * @param hora Hora seleccionada por el usuario.
	 */
	public void guardarDatos(String fecha, String hora);
	
	/**
	 * Obtiene infomacin de la cita que ha ido introduciendo el usuario en la aplicacin.
	 */
	public String obtenerInfoCita();
	
	/**
	 * Confirma la cita que introdujo el usuario en la aplicacin y la envia a la base de datos externa.
	 * @return Informacin de la cita.
	 */
	public void confirmarCita();
	
	/**
	 * Obtiene el listado de citas que ha pedido el usuario a travs de la aplicacin.
	 * @param id_usuario Identificador del usuario.
	 * @return Listado con las citas.
	 */
	public void obtenerListaCitas(String id_usuario);
	
	/**
	 * Obtiene informacin de una cita con su identificador.
	 * @param id_cita Identificador de la cita.
	 */
	public void obtenerInfoCita(String id_cita);
	
	/**
	 * Cancela una cita que ha pedido el usuario y la borra de la base de datos externa.
	 * @return Retorna verdadero si se cancela con xito o falso en caso contrario.
	 */
	public void cancelarCita();
	
	//Aadidos 
	
	/**
	 * Guarda la cita seleccionada en MisCitasMaestro
	 * @param id_cita Identificador de la cita
	 */
	public void setCitaSeleccionada(String id_cita);
	
	/**
	 * Obtiene la cita seleccionada en MisCitasMaestro
	 * @return id_cita Identificador de la cita
	 */
	public String getCitaSeleccionada();
	
	/**
	 * Guarda la peluqueria seleccionada en DondeEstamos
	 * @param id_peluqueria Identificador de la peluqueria
	 */
	public void setPeluqueriaSeleccionada(String id_peluqueria);
	
	/**
	 * Obtiene la peluqueria seleccionada en DondeEstamos
	 * @return id_peluqueria Identificador de la peluqueria
	 */
	public String getPeluqueriaSeleccionada();
	
	/**
	 * Guarda el telefono del usuario
	 * @param telefono Telfono del usuario utilizado como identificador
	 */
	public void setTelefono(String telefono);
	/**
	 * Obtiene el telefono del usuario
	 * @return telefono Telfono del usuario utilizado como identificador
	 */
	public String getTelefono();
	
	/**
	 * Obtiene del servidor externo los datos de todas las peluquerias
	 */
	public void obtenerDatosPeluquerias();
	
	
}
